/* GnoBang: (c) 2008 by Andreas S. Krebs
 * 
 * This file is part of GnoBang.
 *
 * GnoBang is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GnoBang is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GnoBang.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <limits.h>
#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

/* location of some essential files */
#define GNOBANG_UI      INSTALLDIR"/gnobang.ui"
#define GNOBANG_ICON	INSTALLDIR"/gnobang.png"
#define GNOBANG_PREFS	".gnobang"

/* data about the images */
#define FIELD_SIZE		23
#define IMAGE_SIZE		17
#define BORDER_SIZE		6
#define IMAGE_OFFSET	((FIELD_SIZE-IMAGE_SIZE)/2)
#define FIELD_OFFSET  	(IMAGE_SIZE/2 + BORDER_SIZE +1)	

/* the possible values a field can have */
#define FIELD_EMPTY		0
#define FIELD_PLAYER1	1
#define FIELD_PLAYER2	2
#define FIELD_BORDER	3

/* id's for the cheat widgets */
#define UNDO_WIDGET		0
#define REDO_WIDGET		1
#define HINT_WIDGET		2

/* the possible player types */
#define PLAYER_HUMAN	0
#define	PLAYER_COMPUTER	1

#define PLAYER_HINT		2	/* no real player, only for hints */

/* the start player */
#define START_PLAYER1	0
#define START_PLAYER2	1
#define START_LOOSER	2
#define START_WINNER	3

/* game status */
#define GAME_NEW		0
#define GAME_RUNNING	1
#define GAME_PAUSED		2
#define GAME_FINISHED	3

/* the 4 possible directions */
#define DIR_HORIZ	0
#define DIR_LEFTUP	1
#define DIR_VERT	2
#define DIR_RIGHTUP	3

/* play modes */
#define	PLAY_NORMAL		0
#define PLAY_OFFENSIVE	1
#define PLAY_DEFENSIVE	2

/* cheat modes */
#define CHEAT_NONE	0
#define CHEAT_HINT	1
#define CHEAT_UNDO	2

/* the sound files to play */
#define SOUNDS_DIR	INSTALLDIR"/sounds"
#define WIN_SOUND		"success.ogg"
#define LOSE_SOUND		"evillaugh.ogg"
#define OVER_SOUND		"game_over.ogg"
#define NEW_SOUND		"flush.ogg"
#define ILLEGAL_SOUND	"ouch.ogg"
#define GOODBYE_SOUND	"goodbye.ogg"

/* my structs */
typedef struct  
{
	int 	Type;			/* human or computer ? */
	int		PlayMode;		/* play offensive, defensive or normal */
	guint32	Color;			/* color of player and stone */
	char	Name[16];		/* name of player */
	long	Seconds;		/* time played by player */
	time_t	TimeTurnStart;	/* when did the current turn start */
} GoPlayer;

#define OTHER_PLAYER(x)		((x)^1)
typedef struct
{
	GoPlayer	Player[2];			/* 2 players */
	int			PlayersTurn;		/* whose turn ? (player 0 or 1) */
	int			StartPlayer;		/* who starts ? */
	int			LastWinner;			/* who won the last game */
	guint32		BoardColor;			/* color for board grid */
	guint32		BoardBackground;	/* color for board background */
	guint32		HintColor;			/* color for hint stone */
	int 		GameStatus;			/* Status of Game: NEW, PAUSED, RUNNING, FINISHED */ 
	int			Columns, Rows;		/* size of board in Columns and Rows */
	int			Width, Height;		/* size of board in Pixels */
	int			CheatMode;			/* none, hint, undo possible */
	gboolean	SoundOn;			/* play sounds or not */
} GoGame;
  
typedef struct
{
	char	*BoardField;	/* Memory with Rows * Columns fields with status */
	char	*MovesDone;		/* Stack containing all done moves */
	int		MoveNumber;		/* Number of the current move */ 
	int		LastMoveDone;	/* Copy of the highest MoveNumber (for Redo) */
	int		NumberOfFields;	/* = Rows * Columns */
} GoBoard;

typedef struct
{
	int	SameColor;	/* number of chips with same color */
	int	SizeOfArea;	/* number of free fields in direction */
	int	OpenEnds;	/* number of open ends */
	int	SameInArea;	/* chips with same color, but 1 free in between */   
} DirInfo;

typedef struct
{
	/* the widgets for the main window */
	GtkWindow			*window;
	GtkVBox			*mainBox;
	GtkWidget		*drawingArea;
	char			playerColor[2][16];
	GtkLabel		*playerTimer[2];
	GtkLabel		*playerTimerLabel[2];
	GtkLabel		*toMoveLabel;
	GtkAction		*prefsMenuAction;
	GtkImage		*controlImage;
	GtkLabel		*controlLabel;
	GtkButton		*hintButton;
	GtkButton		*undoButton;
	GtkButton		*redoButton;
  
	/* the widgets for the preferences dialog */
	GtkDialog		*prefDiag;
	GtkSpinButton	*columnsButton;
	GtkSpinButton	*rowsButton;
	GtkComboBox		*firstMoveBox;
	GtkComboBox		*cheatModeBox;
	GtkCheckButton	*soundCheckButton;
	GtkComboBox		*playerTypeBox[2];
	GtkEntry		*playerNameField[2];
	GtkComboBox		*playerStrategyBox[2];
	GtkColorButton	*boardColorButton;
	GtkColorButton	*backgroundColorButton;
	GtkColorButton	*hintColorButton;
	GtkColorButton	*playerColorButton[2];
}
GnoBang;

/* global prototypes */

/* widgets.c */
extern void setPlayersTurn(int actPlayer);
extern void startPlayerTimer();
extern void switchOffWidget(int widgetID);	
extern void switchOnWidget(int widgetID);
extern void setGameStatus(int status);
extern void setPlayerTimer(int playerNr);

/* gnobang.c */
extern void showErrorMessage(const gchar *msg);		
extern void applyPreferences();

/* preferences.c */
extern void savePrefs();
extern void loadPrefs();

/* sounds.c */
extern void soundSystemInit(int *pargc, char **pargv[]);
extern void playSound(char *soundname, gboolean block);
extern void soundSystemCleanup();				

/* gobang.c */
extern void newGame();
extern void allocBoard();
extern void freeBoard();
extern int  getField(int x, int y);
extern void setField(int x, int y, int value);
extern void makeMove(int x, int y);

/* gdkgfx.c */
extern void setcolor(char *cs, guint32 col);
extern void setGdkColor(GtkWidget *widget, GdkColor *gdkCol, guint32 col);
extern guint32 getGdkColor(GdkColor *gdkCol);
extern void getShadowColors(guint32 baseCol, guint32 *topCol, guint32 *col, guint32 *botCol);
extern void createXImages();
extern void initGCs();
extern void drawBoard();
extern void displayGameOver(gboolean won, int playerNr, int x1, int y1, int x2, int y2);
extern void drawChipAt(int x, int y, int playerNo);
extern void undoChipAt(int x, int y);
extern void destroyAll();

/* cursor.c */
extern void setCursor(int nr);				
extern void createCursors();
extern void destroyCursors();


/* global variables */
extern GoGame		*goGame;		/* gnobang.c */
extern GnoBang		*gnoBang;		/* gnobang.c */

